import sys
import requests
import logging
import json
import time
import os
import register_lemans_lib
from waiting import wait
from register_lemans_lib import LemansRegistrar
from kubernetes import client, config
from kubernetes.client.rest import ApiException
from requests.auth import HTTPBasicAuth

CSP_URI = sys.argv[1]
MAPPING_FILE_PATH = sys.argv[2]
NAMESPACE = sys.argv[3]


def restart_gateway_pods(namespace):
    api_instance = client.CoreV1Api()
    pod_name = get_pods("lemans-gateway", namespace, api_instance)
    restart_pods(pod_name, namespace, api_instance, "lemans-gateway")


def get_pods(app_label, namespace, api_instance):
    ret = api_instance.list_namespaced_pod(
        namespace, watch=False, label_selector="app=" + app_label
    )
    pod_name = []

    for i in ret.items:
        pod_name.append(i.metadata.name)

    if len(pod_name) == 0:
        raise RuntimeError(
            "No pod exist for app: " + app_label + " in namespace " + namespace
        )
    else:
        print("The pod name for app " + app_label + " is " + str(pod_name))
        return pod_name


def check_all_pods_running(namespace, app_label):
    api_instance = client.AppsV1beta1Api()
    api_response = api_instance.list_namespaced_deployment(
        namespace, watch=False, label_selector="app=" + app_label
    )
    print("Deployment result")
    ready_replicas = int(api_response.items[0].status.ready_replicas)
    expected_replicas = int(api_response.items[0].spec.replicas)
    if expected_replicas == ready_replicas:
        print("Expected number of replicas running")
        return True
    else:
        print("Expected number of replicas not running")
        return False


def restart_pods(pod_names, namespace, api_instance, app_label):
    try:
        for pod_name in pod_names:
            api_response = api_instance.delete_namespaced_pod(pod_name, namespace)
            print("Deleted pod " + str(api_response.metadata.self_link))
            try:
                wait(
                    lambda: check_all_pods_running(namespace, app_label),
                    timeout_seconds=120,
                    waiting_for="Waiting for all pods to be ready",
                )
            except:
                raise Exception("Lemans-gateway pods aren't running")
    except ApiException as e:
        raise Exception("Exception trying to delete pod: %s\n" % e)


logging.getLogger().setLevel(logging.INFO)
config.load_incluster_config()
with open(MAPPING_FILE_PATH) as mapping_file:
    lemans_configurations = json.load(mapping_file)
    registrar = register_lemans_lib.create(NAMESPACE, CSP_URI)
    if registrar is None:
        logging.warning("Skipping registeration with lemans")
        exit(0)

    for _key, configs in lemans_configurations.items():
        for config in configs:
            registrar.register_service_to_lemans(
                stream_name=config["stream_name"], receivers=config["receivers"]
            )
# Restart Lemans gateway pods
RESTART_GATEWAY_PODS = os.getenv("RESTART_GATEWAY_PODS") or "False"
if RESTART_GATEWAY_PODS == "True":
    restart_gateway_pods(NAMESPACE)
